#include <vdr/font.h>

#include "extb.h"
#include "i18n.h"
#include "extb_osd_menu.h"
#include "lirc.h"




class cLEDItem
: public cOsdItem
, public cLircPtr
{
 protected:
  int min_item;
  int max_item;
  static const char* reset_cmd;
  static const char* set_cmd;
  static const char* toggle_cmd;
 public:
  cLEDItem(cLirc* pLirc,int _min_item, int _max_item, const char *name)
  : cLircPtr(pLirc)
  {
    min_item = _min_item;
    max_item = _max_item;
    SetText(name);
  }
  virtual eOSState ProcessKey(eKeys Key);
};

const char* cLEDItem::reset_cmd  = "R_LD%d";
const char* cLEDItem::set_cmd    = "S_LD%d";
const char* cLEDItem::toggle_cmd = "T_LD%d";


eOSState cLEDItem::ProcessKey(eKeys Key)
{
  const char *buff = "";
  eOSState state = cOsdItem::ProcessKey(Key);

  if(kRed == Key)
    {
      buff = reset_cmd;
    }
  else if(kGreen == Key)
    {
      buff = set_cmd;
    }
  else if(kYellow == Key)
    {
      buff = toggle_cmd;
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  if(strlen(buff) > 0)
    {
      for(int i=min_item; i <= max_item; i++)
	{
	  char tmp[6];
	  snprintf(tmp,sizeof(tmp), buff, i);
	  m_pLirc->send(tmp);
	}
    }

  return (state);
}

class cExtOutItem
: public cOsdItem
, public cLircPtr
{
 protected:
  int min_item;
  int max_item;
  static const char* reset_cmd;
  static const char* set_cmd;
  static const char* toggle_cmd;
 public:
  cExtOutItem(cLirc* pLirc,int _min_item, int _max_item, const char *name)
  : cLircPtr(pLirc)
    {
      min_item = _min_item;
      max_item = _max_item;
      SetText(name);
    }
  virtual eOSState ProcessKey(eKeys Key);
};

const char* cExtOutItem::reset_cmd  = "R_OUT%d";
const char* cExtOutItem::set_cmd    = "S_OUT%d";
const char* cExtOutItem::toggle_cmd = "T_OUT%d";

eOSState cExtOutItem::ProcessKey(eKeys Key)
{
  const char *buff = "";
  eOSState state = cOsdItem::ProcessKey(Key);

  if(kRed == Key)
    {
      buff = reset_cmd;
    }
  else if(kGreen == Key)
    {
      buff = set_cmd;
    }
  else if(kYellow == Key)
    {
      buff = toggle_cmd;
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  if(strlen(buff) > 0)
    {
      for(int i=min_item; i <= max_item; i++)
	{
	  char tmp[6];
	  snprintf(tmp,sizeof(tmp), buff, i);
	  m_pLirc->send(tmp);
	}
    }

  return (state);
}

class cStdOnOffItem 
: public cOsdItem
, public cLircPtr
{
 protected:
  char cmd_on[10];
  char cmd_off[10];
 public:
  cStdOnOffItem(cLirc* pLirc, const char* _cmd_on, const char* _cmd_off,const char* const name)
  : cLircPtr(pLirc)
    {
      SetText(name);
      strncpy(cmd_on,  _cmd_on, sizeof(cmd_on));
      strncpy(cmd_off, _cmd_off,sizeof(cmd_off));
    }
  virtual eOSState ProcessKey(eKeys Key);
};

eOSState cStdOnOffItem::ProcessKey(eKeys Key)
{
  const char *buff = "";
  eOSState state = cOsdItem::ProcessKey(Key);

  if(kRed == Key)
    {
      buff = cmd_off;
    }
  else if(kGreen == Key)
    {
      buff = cmd_on;
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  if(strlen(buff) > 0)
    {
      m_pLirc->send(buff);
    }

  return (state);
}

class cMacroItem
: public cOsdItem
, public cLircPtr
{
 protected:
  char cmd[10];
  bool reset_cmd_after_key;
  time_t reset_cmd_time;
 public:
  cMacroItem(cLirc* pLirc,const char* const name, const char* _cmd)
  : cLircPtr(pLirc)
    {
      SetText(name);
      strncpy(cmd, _cmd, sizeof(cmd));
      reset_cmd_after_key = false;
      reset_cmd_time = (time_t)-1;
    }
  virtual eOSState ProcessKey(eKeys Key);
};

eOSState cMacroItem::ProcessKey(eKeys Key)
{
  eOSState state = cOsdItem::ProcessKey(Key);

  if(reset_cmd_after_key)
    {
      time_t now = time(NULL);
      if(now - reset_cmd_time > 15)
	{
	  reset_cmd_after_key = false;
	  m_pLirc->send("M1");
	}
    }

  if(kRed == Key || kGreen == Key || kOk == Key)
    {
      m_pLirc->send(cmd);
      if(kRed == Key)
	{
	  reset_cmd_after_key = true;
	  reset_cmd_time = time(NULL);
	}
    }
  else if(kYellow == Key)
    {
      m_pLirc->send("M1");
    }
  else if(kBlue == Key)
    {
      state = osBack;
    }

  return (state);
}

class cLEDMenu
: public cOsdMenu
{
 public:
  cLEDMenu(cLirc* pLirc);
  virtual eOSState ProcessKey(eKeys Key);
};

cLEDMenu::cLEDMenu(cLirc* pLirc)
  : cOsdMenu(tr("Extension Board - led test"), 10, 10)
{
  SetHasHotkeys();

  Add(new cLEDItem(pLirc,1, 1, hk(tr("LED #1"))));
  Add(new cLEDItem(pLirc,2, 2, hk(tr("LED #2"))));
  Add(new cLEDItem(pLirc,3, 3, hk(tr("LED #3"))));
  Add(new cLEDItem(pLirc,4, 4, hk(tr("LED #4"))));
  Add(new cLEDItem(pLirc,5, 5, hk(tr("LED #5"))));
  Add(new cLEDItem(pLirc,6, 6, hk(tr("LED #6"))));
  Add(new cLEDItem(pLirc,1, 6, hk(tr("LED #1-6"))));

  SetHelp(tr("off"), tr("on"), tr("toggle"), tr("back"));
}

eOSState cLEDMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);
  
  return (state);
}

class cLCDMenu
: public cOsdMenu
{
 public:
  cLCDMenu(cLirc* pLirc);
};

cLCDMenu::cLCDMenu(cLirc* pLirc)
  : cOsdMenu(tr("Extension Board - lcd test"), 10, 10)
{
  SetHasHotkeys();

  Add(new cStdOnOffItem(pLirc, "S_LCD",    "R_LCD",   hk(tr("LCD contract voltage"))));
  Add(new cStdOnOffItem(pLirc, "S_LCD_BL", "R_LCD_BL",hk(tr("LCD backlight"))));

  SetHelp(tr("off"), tr("on"), NULL, tr("back"));
}

class cExtOutMenu
: public cOsdMenu
{
 public:
  cExtOutMenu(cLirc* pLirc);
};

cExtOutMenu::cExtOutMenu(cLirc* pLirc)
  : cOsdMenu(tr("Extension Board - external output"), 10, 10)
{
  SetHasHotkeys();

  Add(new cExtOutItem(pLirc,1, 1, hk(tr("Output #1"))));
  Add(new cExtOutItem(pLirc,2, 2, hk(tr("Output #2"))));
  Add(new cExtOutItem(pLirc,3, 3, hk(tr("Output #3"))));
  Add(new cExtOutItem(pLirc,4, 4, hk(tr("Output #4"))));
  Add(new cExtOutItem(pLirc,1, 4, hk(tr("Output #1-4"))));

  SetHelp(tr("off"), tr("on"), tr("toggle"), tr("back"));
}

class cMacroMenu
: public cOsdMenu
{
 public:
  cMacroMenu(cLirc* pLirc);
};

cMacroMenu::cMacroMenu(cLirc* pLirc)
  : cOsdMenu(tr("Extension Board - macros"), 10, 10)
{
  SetHasHotkeys();

  Add(new cMacroItem(pLirc,hk(tr("Macro #1")), "M1"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #2")), "M2"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #3")), "M3"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #4")), "M4"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #5")), "M5"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #6")), "M6"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #7")), "M7"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #8")), "M8"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #9")), "M9"));
  Add(new cMacroItem(pLirc,hk(tr("Macro #10")), "M10"));

  SetHelp(tr("test 15s"), tr("send"), tr("reset (M1)"), tr("back"));
}

class cVideoMenu
: public cOsdMenu
{
 public:
  cVideoMenu(cLirc* pLirc);
};

cVideoMenu::cVideoMenu(cLirc* pLirc)
  : cOsdMenu(tr("Extension Board - video"), 10, 10)
{
  SetHasHotkeys();

  Add(new cMacroItem(pLirc,hk(tr("Videoquelle DVB")), "S_DVB"));
  Add(new cMacroItem(pLirc,hk(tr("Videoquelle VGA")), "S_VGA"));
  Add(new cMacroItem(pLirc,hk(tr("Videoquelle AUX")), "S_AUX"));
  Add(new cMacroItem(pLirc,hk(tr("Videoquelle aus")), "R_VID_ALL"));
  Add(new cMacroItem(pLirc,hk(tr("Videoquelle DVB/VGA umschalten")), "T_DVB_VGA"));
  Add(new cMacroItem(pLirc,hk(tr("Videoquelle DVB/AUX umschalten")), "T_DVB_AUX"));
  Add(new cMacroItem(pLirc,hk(tr("Videoquelle VGA/AUX umschalten")), "T_VGA_AUX"));
  Add(new cMacroItem(pLirc,hk(tr("Scart #8 6V")), "S_SC8_6V"));
  Add(new cMacroItem(pLirc,hk(tr("Scart #8 12V")), "R_SC8_6V"));

  SetHelp(tr("test 15s"), tr("send"), tr("reset (M1)"), tr("back"));
}

class cAudioMenu
: public cOsdMenu
{
 public:
  cAudioMenu(cLirc* pLirc);
};

cAudioMenu::cAudioMenu(cLirc* pLirc)
  : cOsdMenu(tr("Extension Board - audio"), 10, 10)
{
  SetHasHotkeys();

  Add(new cMacroItem(pLirc,hk(tr("analog Audio an")), "S_ANA"));
  Add(new cMacroItem(pLirc,hk(tr("analog Audio aus")), "R_ANA"));
  Add(new cMacroItem(pLirc,hk(tr("analog Audio umschalten")), "T_ANA"));
  Add(new cMacroItem(pLirc,hk(tr("digital Audio an")), "S_DIA"));
  Add(new cMacroItem(pLirc,hk(tr("digital Audio aus")), "R_DIA"));
  Add(new cMacroItem(pLirc,hk(tr("digital Audio umschalten")), "T_DIA"));
  Add(new cMacroItem(pLirc,hk(tr("analog+digital Audio an")), "S_ANDIA"));
  Add(new cMacroItem(pLirc,hk(tr("analog+digital Audio aus")), "R_ANDIA"));
  Add(new cMacroItem(pLirc,hk(tr("analog+digital Audio umschalten")), "T_ANDIA"));

  SetHelp(tr("test 15s"), tr("send"), tr("reset (M1)"), tr("back"));
}

cExtBOsdMenu::cExtBOsdMenu(cLirc* pLirc)
 : cOsdMenu(tr("Extension Board"))
 , cLircPtr(pLirc)
{
  SetHasHotkeys();

  Add(new cOsdItem(hk(tr("Status LEDs")),              osUser1));
  Add(new cOsdItem(hk(tr("LC-Display & Backlight")),   osUser2));
  Add(new cOsdItem(hk(tr("External Outputs")),         osUser3));
  Add(new cOsdItem(hk(tr("Macros")),                   osUser4));
  Add(new cOsdItem(hk(tr("Audio")),                    osUser5));
  Add(new cOsdItem(hk(tr("Video")),                    osUser6));

  SetHelp(NULL, NULL, NULL, tr("back"));
  Display();
}

cExtBOsdMenu::~cExtBOsdMenu()
{
}

eOSState cExtBOsdMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  switch(state)
    {
    case osUser1: 
      state = AddSubMenu(new cLEDMenu(m_pLirc));
      break;
      
    case osUser2: 
      state = AddSubMenu(new cLCDMenu(m_pLirc));
      break;
      
    case osUser3: 
      state = AddSubMenu(new cExtOutMenu(m_pLirc));
      break;
      
    case osUser4: 
      state = AddSubMenu(new cMacroMenu(m_pLirc));
      break;
      
    case osUser5: 
      state = AddSubMenu(new cAudioMenu(m_pLirc));
      break;

    case osUser6: 
      state = AddSubMenu(new cVideoMenu(m_pLirc));
      break;

    case osContinue:
      break;

    default:
      if(kBlue == Key)
	{
	  state = osBack;
	}
    }

  if(!HasSubMenu())
    Display();

  return (state);
}
