
#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif 
#include <stdio.h>

#include "extb.h"
#include "i18n.h"
#include "extb_setup_menu.h"
#include "setup.h"

cExtBMenuSetup::cExtBMenuSetup(cExtBoardIO*    pExt)
     //  : cOsdMenu(tr("Extension Board"), 20, 20)
 : m_pExt(pExt)
 , m_tempSetup(theSetup)
 , wait(0)
 , init_menue(true)
{
  unsigned int n;

  m_nOldVideoModeDVB = 
    m_nVideoModeDVB = eRGB;
  m_nOldVideoModeVGA = 
    m_nVideoModeVGA = eRGB;
  m_nOldVideoModeAUX = 
    m_nVideoModeAUX = eRGB;

  m_nOldScartPin8Mode = 
     m_nScartPin8Mode = eF_SC8_PWR;
  m_nOldVideoSourceOnBoot =
     m_nVideoSourceOnBoot = eF_BOOT_VGA;   

  for(n=0;n<memberof(m_szVideoModesDVB);++n) 
    m_szVideoModesDVB[n] = NULL;
  
  for(n=0;n<memberof(m_szVideoModesAUX);++n) 
    m_szVideoModesAUX[n] = NULL;
  
  for(n=0;n<memberof(m_szVideoModesVGA);++n) 
    m_szVideoModesVGA[n] = NULL;
  
  for(n=0;n<memberof(m_szScartPin8Mode);++n) 
    m_szScartPin8Mode[n] = NULL;
  
  for(n=0;n<memberof(m_szVideoSourceOnBoot);++n) 
    m_szVideoSourceOnBoot[n] = NULL;
  
  gLircWatch->StartWatch();
  m_pExt->send("STATUS");
#if VDRVERSNUM >= 10307
  Skins.Message(mtInfo,tr("reading settings from controller, please wait"));
#else
  Interface->Info(tr("reading settings from controller, please wait"));
#endif

  m_tempSetup.LoadTranslatedStings();
  
  SetHelp(NULL, NULL, NULL, NULL); // Use only default keys, OK-> Store, Exit -> Back
}

cExtBMenuSetup::~cExtBMenuSetup()
{
  unsigned int n;
  for(n=0;n<memberof(m_szVideoModesDVB);++n) 
    if(m_szVideoModesDVB[n]) 
      free(m_szVideoModesDVB[n]);
    
  for(n=0;n<memberof(m_szVideoModesAUX);++n)
    if(m_szVideoModesAUX[n]) 
      free(m_szVideoModesAUX[n]);
    
  for(n=0;n<memberof(m_szVideoModesVGA);++n)
    if(m_szVideoModesVGA[n]) 
      free(m_szVideoModesVGA[n]);
    
  for(n=0;n<memberof(m_szScartPin8Mode);++n)
    if(m_szScartPin8Mode[n]) 
      free(m_szScartPin8Mode[n]);
    
  for(n=0;n<memberof(m_szVideoSourceOnBoot);++n)
    if(m_szVideoSourceOnBoot[n]) 
      free(m_szVideoSourceOnBoot[n]);
}

// Translationtable LIRC-Code to Scart and Bootmode
const cExtBMenuSetup::ScartBootMode cExtBMenuSetup::ScartBootModes[] = {
  //  ScartPin8Mode  VideoSourceOnBoot 
  {   eF_SC8_OFF,    eF_BOOT_OFF  },// SB0
  {   eF_SC8_PWR,    eF_BOOT_OFF  },// SB1
  {   eF_SC8_VDR,    eF_BOOT_OFF  },// SB2
  {  -1,            -1            },// don't use this line
  {   eF_SC8_OFF,    eF_BOOT_VGA  },// SB4
  {   eF_SC8_PWR,    eF_BOOT_VGA  },// SB5
  {   eF_SC8_VDR,    eF_BOOT_VGA  },// SB6
  {  -1,            -1            },// don't use this line
  {   eF_SC8_OFF,    eF_BOOT_AUX  },// SB8
  {   eF_SC8_PWR,    eF_BOOT_AUX  },// SB9
  {   eF_SC8_VDR,    eF_BOOT_AUX  },// SBa
};
             
// Translationtable LIRC-Code to Videomode
const cExtBMenuSetup::VideoMode cExtBMenuSetup::VideoModes[] = {
  //  DVB    VGA    AUX
  {   eSVID, eSVID, eSVID  },// V0
  {   eRGB,  eSVID, eSVID  },// V1
  {   eSVID, eRGB,  eSVID  },// V2
  {   eRGB,  eRGB,  eSVID  },// V3
  {   eSVID, eSVID, eRGB   },// V4
  {   eRGB,  eSVID, eRGB   },// V5
  {   eSVID, eRGB,  eRGB   },// V6
  {   eRGB,  eRGB,  eRGB   } // V7
};
             
eOSState cExtBMenuSetup::ProcessKey(eKeys Key)
{
  char sz[256];
  eOSState state;
  state = cMenuSetupPage::ProcessKey(Key);

  if(!gLircWatch->WatchSuccess())
  {
    ++wait;
    if(wait < 5)
    {
      strncpy(sz,tr("reading settings from controller, please wait"),sizeof(sz));
      for(int i=0; i < wait; i++)
        strncat(sz, ".",sizeof(sz));
    
#if VDRVERSNUM >= 10307
      Skins.Message(mtInfo,sz);
#else
	    Interface->Info(sz);
#endif
    }
    else {
#if VDRVERSNUM >= 10307
      Skins.Message(mtError,tr("can't read settings from controller"));
#else
      Interface->Error(tr("can't read settings from controller"));
#endif
    }
  }
  else if(init_menue)
  {
    init_menue = false;
    
    // Translate LIRC-Code to Videomode      
    unsigned int nV = gLircWatch->GetVideoModes();
    if(nV<memberof(VideoModes)) {
      m_nOldVideoModeDVB = 
        m_nVideoModeDVB = VideoModes[nV].nDVB;
      m_nOldVideoModeVGA = 
        m_nVideoModeVGA = VideoModes[nV].nVGA;
      m_nOldVideoModeAUX = 
        m_nVideoModeAUX = VideoModes[nV].nAUX;
    }
    
    // Translate LIRC-Code to Scart and Bootmode 
    unsigned int nSB = gLircWatch->GetScartBootModes();
    if(nSB<memberof(ScartBootModes) 
      && ScartBootModes[nSB].nScartPin8Mode != -1
      && ScartBootModes[nSB].nVideoSourceOnBoot != -1) {
        m_nOldScartPin8Mode = 
           m_nScartPin8Mode = ScartBootModes[nSB].nScartPin8Mode;
        m_nOldVideoSourceOnBoot =
           m_nVideoSourceOnBoot = ScartBootModes[nSB].nVideoSourceOnBoot;        
      }
      
    // assign read values from controller with current visble active mode => (X)
    asprintf(&m_szVideoModesDVB[eRGB],"%s%s",tr("RGB"),m_nVideoModeDVB==eRGB?" (X)":""); 
    asprintf(&m_szVideoModesVGA[eRGB],"%s%s",tr("RGB"),m_nVideoModeVGA==eRGB?" (X)":""); 
    asprintf(&m_szVideoModesAUX[eRGB],"%s%s",tr("RGB"),m_nVideoModeAUX==eRGB?" (X)":""); 
    
    asprintf(&m_szVideoModesDVB[eSVID],"%s%s",tr("S-Video/comp. sync"),  m_nVideoModeDVB==eSVID?" (X)":""); 
    asprintf(&m_szVideoModesVGA[eSVID],"%s%s",tr("S-Video/comp. sync"),  m_nVideoModeVGA==eSVID?" (X)":""); 
    asprintf(&m_szVideoModesAUX[eSVID],"%s%s",tr("S-Video/comp. sync"),  m_nVideoModeAUX==eSVID?" (X)":""); 
    
    asprintf(&m_szScartPin8Mode[eF_SC8_PWR],"%s%s",tr("Power on"),    m_nScartPin8Mode==eF_SC8_PWR?" (X)":""); 
    asprintf(&m_szScartPin8Mode[eF_SC8_VDR],"%s%s",tr("running VDR"), m_nScartPin8Mode==eF_SC8_VDR?" (X)":""); 
    asprintf(&m_szScartPin8Mode[eF_SC8_OFF],"%s%s",tr("off"),         m_nScartPin8Mode==eF_SC8_OFF?" (X)":""); 

    asprintf(&m_szVideoSourceOnBoot[eF_BOOT_VGA],"%s%s",tr("VGA"),    m_nVideoSourceOnBoot==eF_BOOT_VGA?" (X)":""); 
    asprintf(&m_szVideoSourceOnBoot[eF_BOOT_AUX],"%s%s",tr("AUX"),    m_nVideoSourceOnBoot==eF_BOOT_AUX?" (X)":""); 
    asprintf(&m_szVideoSourceOnBoot[eF_BOOT_OFF],"%s%s",tr("off"),    m_nVideoSourceOnBoot==eF_BOOT_OFF?" (X)":""); 
  
      
    // Create all menuentrys
    Add(new cMenuEditBoolItem(tr("Hide Mainmenu Entry"),        
            &m_tempSetup.m_bHideMainMenuEntry,       tr("no"),tr("yes")));

    Add(new cMenuEditStraItem(tr("Deactivate on mute"),  
            &m_tempSetup.m_nMuteMode, 
            memberof(m_tempSetup.m_szMuteModes), 
            m_tempSetup.m_szMuteModes));

    for(unsigned int n=0;n<memberof(m_tempSetup.m_nLEDMode);++n) {
      snprintf(sz,sizeof(sz),tr("Activate LED #%d"),n + 1);

      Add(new cMenuEditStraItem(sz,&m_tempSetup.m_nLEDMode[n], 
              memberof(m_tempSetup.m_szLEDModes), 
              m_tempSetup.m_szLEDModes));
    }
    
    for(unsigned int n=0;n<memberof(m_tempSetup.m_nOutputMode);++n) {
      snprintf(sz,sizeof(sz),tr("Activate ext. Output #%d"),n + 1);

      Add(new cMenuEditStraItem(sz,&m_tempSetup.m_nOutputMode[n], 
              memberof(m_tempSetup.m_szLEDModes), 
              m_tempSetup.m_szLEDModes));
    }
    
    Add(new cMenuEditStraItem(hk(tr("DVB mode")),              
      &m_nVideoModeDVB, 
      memberof(m_szVideoModesDVB), m_szVideoModesDVB));

    Add(new cMenuEditStraItem(hk(tr("VGA mode")),              
      &m_nVideoModeVGA, 
      memberof(m_szVideoModesVGA), m_szVideoModesVGA));

    Add(new cMenuEditStraItem(hk(tr("AUX mode")),              
      &m_nVideoModeAUX, 
      memberof(m_szVideoModesAUX), m_szVideoModesAUX));

    Add(new cMenuEditStraItem(hk(tr("Enable Scart pin #8")),   
      &m_nScartPin8Mode, 
      memberof(m_szScartPin8Mode), m_szScartPin8Mode));

    Add(new cMenuEditStraItem(hk(tr("Video source on boot")),  
      &m_nVideoSourceOnBoot, 
      memberof(m_szVideoSourceOnBoot), m_szVideoSourceOnBoot));
    
#if VDRVERSNUM >= 10307
   //Skins.Message(mtInfo,NULL);
#else
    Interface->Status(NULL);
#endif
    Display();
  }
  return (state);
}


/// Store Profile on leave Setupmenu with OK
void cExtBMenuSetup::Store()
{
  theSetup = m_tempSetup;
  // HideMainMenuEntry
  SetupStore("HideMainMenuEntry", 		theSetup.m_bHideMainMenuEntry);

  // MuteMode
  SetupStore("MuteMode", 				      theSetup.m_nMuteMode);

  // store Setup for all LEDs
  char sz[16];//LEDMode1,LEDMode2...
  for(unsigned int n=0;n<memberof(theSetup.m_nLEDMode);++n) {
      snprintf(sz,sizeof(sz),"LEDMode%d",n + 1);  
      SetupStore(sz, 				          theSetup.m_nLEDMode[n]); 
  }
  // store Setup for all Outputs (Output1,Output2...)
  for(unsigned int n=0;n<memberof(theSetup.m_nOutputMode);++n) {
      snprintf(sz,sizeof(sz),"Output%d",n + 1);  
      SetupStore(sz, 				          theSetup.m_nOutputMode[n]); 
  }
  
  if((m_nOldVideoModeDVB != m_nVideoModeDVB) // send only if changed
     || (m_nOldVideoModeVGA != m_nVideoModeVGA) 
     || (m_nOldVideoModeVGA != m_nVideoModeVGA) 
     || (m_nOldScartPin8Mode != m_nScartPin8Mode)  
     || (m_nOldVideoSourceOnBoot != m_nVideoSourceOnBoot)) {
#if VDRVERSNUM >= 10307
      Skins.Message(mtInfo,tr("send settings to controller"));
#else
      Interface->Info(tr("send settings to controller"));
#endif
  
    if(m_nOldVideoModeDVB != m_nVideoModeDVB) {
      switch(m_nVideoModeDVB) {
      case eRGB:  m_pExt->send("F_DVB_RGB"); break;
      case eSVID: m_pExt->send("F_DVB_CS");  break;
      }
    }
    
    if(m_nOldVideoModeVGA != m_nVideoModeVGA) {
      switch(m_nVideoModeVGA) {
      case eRGB:  m_pExt->send("F_VGA_RGB"); break;
      case eSVID: m_pExt->send("F_VGA_CS");  break;
      }
    }
    
    if(m_nOldVideoModeVGA != m_nVideoModeVGA) {
      switch(m_nVideoModeAUX) {
      case eRGB:  m_pExt->send("F_AUX_RGB"); break;
      case eSVID: m_pExt->send("F_AUX_CS");  break;
      }
    }
    
    // Set which Mode is used for Scart Pin 8
    if(m_nOldScartPin8Mode != m_nScartPin8Mode) { // send only if changed
      switch(m_nScartPin8Mode) {
        case eF_SC8_PWR: m_pExt->send("F_SC8_ON_PWR"); break;
        case eF_SC8_VDR: m_pExt->send("F_SC8_ON_VDR"); break;
        case eF_SC8_OFF: m_pExt->send("F_SC8_OFF"); break;
      }
    }
    
    // Set which Source is used during boot
    if(m_nOldVideoSourceOnBoot != m_nVideoSourceOnBoot) { // send only if changed
      switch(m_nVideoSourceOnBoot) {
        case eF_BOOT_VGA: m_pExt->send("F_BOOT_VGA"); break;
        case eF_BOOT_AUX: m_pExt->send("F_BOOT_AUX"); break;
        case eF_BOOT_OFF: m_pExt->send("F_BOOT_OFF"); break;
      }
    }
  }
  
  m_pExt->Update(true);
}
