/*
 * extb plugin to VDR (C++)
 *
 * (C) 2004 Andreas Brachold <vdr04-at-deltab-dot-de>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <vdr/tools.h>
#include <vdr/config.h>

#include "extboardio.h"
#include "lirc.h"


/// Ctor, load default values
cExtBoardIO::cExtBoardIO()
{
  unsigned int n;
  for(n=0;n<memberof(m_nLastSendLEDState);++n) {
      m_nLastSendLEDState[n] = -1;  
  }  
  for(n=0;n<memberof(m_nLastSendOutputState);++n) {
      m_nLastSendOutputState[n] = -1;  
  }  
  for(n=0;n<memberof(m_nCardIsRecording);++n) {
      m_nCardIsRecording[n] = 0;  
  }
  m_eReplay = eReplayNone;
  m_eDVDAudio = eDVDNone;
  m_eDVDVideo = eDVDVideoNone;
  m_nMP3 = 0;

  m_nChannel = 0;
  
  m_nLastVolume = cDevice::CurrentVolume();
}

//ext. Output///////////////////////////////////////////////////////////////////  
const char * cExtBoardIO::m_szOutput_ON [OUT_CNT] = 
{
  "S_OUT1",
  "S_OUT2",
  "S_OUT3",
  "S_OUT4"
};
const char * cExtBoardIO::m_szOutput_OFF[OUT_CNT] =
{
  "R_OUT1",
  "R_OUT2",
  "R_OUT3",
  "R_OUT4"
};

//LED///////////////////////////////////////////////////////////////////////////  
const char * cExtBoardIO::m_szLED_ON [LED_CNT] = 
{
  "S_LD1",
  "S_LD2",
  "S_LD3",
  "S_LD4",
  "S_LD5",
  "S_LD6"
};
const char * cExtBoardIO::m_szLED_OFF[LED_CNT] =
{
  "R_LD1",
  "R_LD2",
  "R_LD3",
  "R_LD4",
  "R_LD5",
  "R_LD6"
};


bool cExtBoardIO::ShouldOn(int nModes) const
{
  unsigned int n;
  switch(nModes)
  {
  // Instant on or off
  case cExtbSetup::eLEDModes_Off:             return false;
  case cExtbSetup::eLEDModes_AlwaysOn:        return true;
  // Recording 
  case cExtbSetup::eLEDModes_AnyRecording: {
    for(n=0;n<memberof(m_nCardIsRecording);++n) {
        if(0 != m_nCardIsRecording[n])
         return true; 
    }    
    return false;
  }
  case cExtbSetup::eLEDModes_RecordingOnCard1: return (0 != m_nCardIsRecording[0]);
  case cExtbSetup::eLEDModes_RecordingOnCard2: return (0 != m_nCardIsRecording[1]);
  case cExtbSetup::eLEDModes_RecordingOnCard3: return (0 != m_nCardIsRecording[2]);
  case cExtbSetup::eLEDModes_RecordingOnCard4: return (0 != m_nCardIsRecording[3]);
  // Replay 
  case cExtbSetup::eLEDModes_AnyReplay:        return m_eReplay != eReplayNone;
  case cExtbSetup::eLEDModes_ReplayVDR:        return m_eReplay == eReplayVDR;
  case cExtbSetup::eLEDModes_ReplayDVD:        return m_eReplay == eReplayDVD;
  case cExtbSetup::eLEDModes_ReplayMP3:        return m_eReplay == eReplayMP3;
  // Channelfeature
  case cExtbSetup::eLEDModes_ChannelHasCA:     return (m_eReplay == eReplayNone) && (m_nChannel & eHasCA);
  case cExtbSetup::eLEDModes_ChannelHasDpid:   return (m_eReplay == eReplayNone) && (m_nChannel & eHasDpid);
  case cExtbSetup::eLEDModes_ChannelHasApid:   return (m_eReplay == eReplayNone) && (m_nChannel & eHasApid);
  case cExtbSetup::eLEDModes_ChannelHasTpid:   return (m_eReplay == eReplayNone) && (m_nChannel & eHasTpid);
  case cExtbSetup::eLEDModes_ChannelHasVpid:   return (m_eReplay == eReplayNone) && (m_nChannel & eHasVpid);
  case cExtbSetup::eLEDModes_ChannelIsRadio:   return (m_eReplay == eReplayNone) && (m_nChannel & eIsRadio);
  // Audio state
  case cExtbSetup::eLEDModes_AudioMute:        return IsMute();
  case cExtbSetup::eLEDModes_AudioNotMute:     return !IsMute();
  // DVD Plugin Playbackmodes
  case cExtbSetup::eLEDModes_DVD_AC3:          return m_eReplay == eReplayDVD && m_eDVDAudio == eDVDAC3;
  case cExtbSetup::eLEDModes_DVD_DTS:          return m_eReplay == eReplayDVD && m_eDVDAudio == eDVDDTS;
  case cExtbSetup::eLEDModes_DVD_PCM:          return m_eReplay == eReplayDVD && m_eDVDAudio == eDVDPCM;
  case cExtbSetup::eLEDModes_DVD_MP2:          return m_eReplay == eReplayDVD && m_eDVDAudio == eDVDMP2;
  case cExtbSetup::eLEDModes_DVD_4_3:          return m_eReplay == eReplayDVD && m_eDVDVideo == eDVDVideo4_3;
  case cExtbSetup::eLEDModes_DVD_16_9:         return m_eReplay == eReplayDVD && m_eDVDVideo == eDVDVideo16_9;
  // MP3 Plugin Playbackmodes
  case cExtbSetup::eLEDModes_MP3_Shuffle:      return m_eReplay == eReplayMP3 && (m_nMP3 & eMP3Shuffle);
  case cExtbSetup::eLEDModes_MP3_Loop:         return m_eReplay == eReplayMP3 && (m_nMP3 & eMP3Loop);
  
  default:
    return false;
  }
}

void cExtBoardIO::Update(bool bForce)
{
  unsigned int n;
  
  //Update LED if changed 
  for(n=0;n<memberof(theSetup.m_nLEDMode);++n) {
    if(theSetup.m_nLEDMode[n] != cExtbSetup::eLEDModes_Unused) {
      int nShouldLEDOn = ShouldOn(theSetup.m_nLEDMode[n])?1:0;
  
      if(bForce || m_nLastSendLEDState[n] != nShouldLEDOn) {
        m_nLastSendLEDState[n] = nShouldLEDOn;
        if(nShouldLEDOn)
          cLirc::send(m_szLED_ON[n]);
        else
          cLirc::send(m_szLED_OFF[n]);
      }
    }
  }
 
  //Update Output if changed 
  for(n=0;n<memberof(theSetup.m_nOutputMode);++n) {
    if(theSetup.m_nOutputMode[n] != cExtbSetup::eLEDModes_Unused) {
      int nShouldOutputOn = ShouldOn(theSetup.m_nOutputMode[n])?1:0;
  
      if(bForce || m_nLastSendOutputState[n] != nShouldOutputOn) {
        m_nLastSendOutputState[n] = nShouldOutputOn;
        if(nShouldOutputOn)
          cLirc::send(m_szOutput_ON[n]);
        else
          cLirc::send(m_szOutput_OFF[n]);
      }
    }
  }
}


void cExtBoardIO::Recording(const cDevice *pDevice, const char *szName)
{
  unsigned int nCardIndex = pDevice->CardIndex();
  if (nCardIndex > memberof(m_nCardIsRecording) - 1 )
    nCardIndex = memberof(m_nCardIsRecording)-1;

  if (nCardIndex < memberof(m_nCardIsRecording)) {
    if (szName) {
      ++m_nCardIsRecording[nCardIndex];
    }
    else {
      if (m_nCardIsRecording[nCardIndex] > 0)
        --m_nCardIsRecording[nCardIndex];
    }
  }
  else {
    esyslog("extb: Recording: only up to %d devices are supported by this plugin", memberof(m_nCardIsRecording));
  }
  Update(false);
}

void cExtBoardIO::Replaying(const cControl *pControl, const char *szName)
{
  if(szName) {
    m_eReplay = eReplayVDR;
    bool bFound = false;
    ///////////////////////////////////////////////////////////////////////
    //Looking for mp3-Plugin Replay : [LS] (444/666) title
    //
    if(strlen(szName)>6
      && *(szName+0)=='['
      && *(szName+3)==']'
      && *(szName+5)=='(')
    {
      unsigned int i;
      for(i=6;*(szName+i) != '\0';++i) { //search for [xx] (xxxx) title
        if(*(szName+i)==' ' && *(szName+i-1)==')') {
          bFound = true;
          break;
        }
      }
      if(bFound) //found MP3-Plugin replaymessage
      {
        m_eReplay = eReplayMP3;
        m_nMP3 = 0;
        m_nMP3 |= ((*(szName+1)) == 'L')?eMP3Loop:0;
        m_nMP3 |= ((*(szName+2)) == 'S')?eMP3Shuffle:0;
      }
    }
    ///////////////////////////////////////////////////////////////////////
    //Looking for DVD-Plugin Replay : 1/8 4/28,  de 2/5 ac3, no 0/7,  16:9, VOLUMENAME
    //cDvdPlayerControl::GerDisplayHeaderLine
    //                         titleinfo, audiolang,  spulang, aspect, title
    if(!bFound) {
      if(strlen(szName)>7) {
      unsigned int i,n;
      for(n=0,i=0;*(szName+i) != '\0';++i) {
      //search volumelabel after 4*", " => xxx, xxx, xxx, xxx, title
        if(*(szName+i)==' ' && *(szName+i-1)==',') {
          if(++n == 4) {
            bFound = true;
            break;
          }
        }
      }    
      if(bFound) //found DVD-Plugin replaymessage
      {
        m_eReplay = eReplayDVD;

        if((strstr(szName,"ac3") != NULL))
          m_eDVDAudio = eDVDAC3;
        else if((strstr(szName,"dts") != NULL))
          m_eDVDAudio = eDVDDTS;
        else if((strstr(szName,"pcm") != NULL))
          m_eDVDAudio = eDVDPCM;
        else if((strstr(szName,"mp2") != NULL))
          m_eDVDAudio = eDVDMP2;
        else
          m_eDVDAudio = eDVDNone;
        
        if((strstr(szName,"16:9") != NULL))
          m_eDVDVideo = eDVDVideo4_3;
        else if((strstr(szName,"4:3") != NULL))
          m_eDVDVideo = eDVDVideo16_9;
        else
          m_eDVDVideo = eDVDVideoNone;
      }
      }
    }
    if (!bFound) {
      m_eReplay = eReplayVDR;
    }
  }
  else
  {
    m_eReplay = eReplayNone;
  }
  Update(false);
}

void cExtBoardIO::Channel(int nChannelNumber)
{
  if(nChannelNumber > 0) {
    const cChannel * pChannel = Channels.GetByNumber(nChannelNumber);
    if(pChannel && !pChannel->GroupSep()) {
      m_nChannel = 0; 
      m_nChannel  |= (0 != pChannel->Ca())?eHasCA:0;
      m_nChannel  |= (0 != pChannel->Dpid1())?eHasDpid:0;
      m_nChannel  |= (0 != pChannel->Apid2())?eHasApid:0;
      m_nChannel  |= (0 != pChannel->Vpid()) ?eHasVpid:eIsRadio;//if channel has no vpid, should be a radiostation
      m_nChannel  |= (0 != pChannel->Tpid()) ?eHasTpid:0;
    }
    else {
      m_nChannel = 0;
    }
  Update(false);
  }
}


//AUDIO/////////////////////////////////////////////////////////////////////////
const char * cExtBoardIO::m_szAnalog [2] = {
  "R_ANA",
  "S_ANA"
};

const char * cExtBoardIO::m_szDigital [2] = {
  "D_DIA",
  "S_DIA"
};

const char * cExtBoardIO::m_szAnalogAndDigital [2] = {
  "R_ANDIA",
  "S_ANDIA"
};


void cExtBoardIO::SendMute(unsigned int nMuteMode,bool bMute)
{
  const char* szSend = NULL;
  switch(nMuteMode)
  {
    case cExtbSetup::eMuteModes_Analog:       szSend = m_szAnalog[bMute?0:1];   
    case cExtbSetup::eMuteModes_Digital:      szSend = m_szDigital[bMute?0:1];   
    case cExtbSetup::eMuteModes_AnalogDigital:szSend = m_szAnalogAndDigital[bMute?0:1];   
  }
  if(szSend)
    cLirc::send(szSend);
}

bool cExtBoardIO::Volume(int nVolume)
{
  bool bStateIsChanged = false;
  if(m_nLastVolume > 0 && 0 == nVolume) {
    SendMute(theSetup.m_nMuteMode,true);
    bStateIsChanged = true;
  }
  else if(0 == m_nLastVolume && nVolume > 0) {
    SendMute(theSetup.m_nMuteMode,false);
    bStateIsChanged = true;
  }
  m_nLastVolume = nVolume;
  return bStateIsChanged;
}
